@@ -1,8 +1,11 @@
-__version__ = "0.490"
+__version__ = "0.491"
 from .loader import *
 from .charts import *
 from .paths import *
 from .markup import *
+from .inspector import *
+from .pdf_loader import PDF
+
 try:
     from .torch_loader import *
 except Exception as e:
