@@ -63,8 +63,9 @@ __all__ = [
     "reset_logger_width",
     "display",
     "typedispatch",
-    "deepcopy",
-    "batchify",
+    "defaultdict",
+    "Counter",
+    "dcopy",
 ]
 
 from .logger import *
@@ -73,8 +74,6 @@ from fastcore.foundation import L
 from fastcore.dispatch import typedispatch
 
 import glob, numpy as np, pandas as pd, tqdm, os, sys, re
-from copy import deepcopy
-from typing import Union, List, Tuple
 from IPython.display import display
 import PIL
 from PIL import Image
@@ -93,6 +92,7 @@ import matplotlib  # ; matplotlib.use('Agg')
 import matplotlib.pyplot as plt
 import matplotlib.patheffects as path_effects
 import pdb, datetime
+from typing import Union, Tuple
 
 E = enumerate
 
@@ -104,6 +104,8 @@ except:
     logger.warning("Skipping cv2 import")
 
 import time
+from collections import defaultdict, Counter
+from copy import deepcopy as dcopy
 
 
 class Timer:
@@ -242,19 +244,19 @@ def C(im):
 
 def common(a, b):
     """Wrapper around set intersection"""
-    x = list(set(a).intersection(set(b)))
+    x = set(a).intersection(set(b))
     logger.opt(depth=1).log(
         "INFO",
         f"{len(x)} items found common from containers of {len(a)} and {len(b)} items respectively",
     )
-    return sorted(x)
+    return set(sorted(x))
 
 
 def diff(a, b, rev=False, silent=False):
     if not rev:
-        o = list(sorted(set(a) - set(b)))
+        o = set(sorted(set(a) - set(b)))
     else:
-        o = list(sorted(set(b) - set(a)))
+        o = set(sorted(set(b) - set(a)))
     if not silent:
         logger.opt(depth=1).log("INFO", f"{len(o)} items found to differ")
     return o
@@ -306,6 +308,7 @@ def show(
             img = img.cpu().detach().numpy().copy()
         if isinstance(img, PIL.Image.Image):
             img = np.array(img)
+
     except Exception as e:
         print(e)
     if not isinstance(img, np.ndarray):
@@ -419,8 +422,9 @@ def show(
                 head_length=4,
                 width=meta * 2,
             )
-            puttext(ax, f"{meta:.2f}", (_xc, _yc), size=text_sz)
-
+            if kwargs.get("conn_text", True):
+                puttext(ax, f"{meta:.2f}", (_xc, _yc), size=text_sz)
+        kwargs.pop("conn_text")
     ax.imshow(img, cmap=cmap, **kwargs)
 
     if grid:
@@ -461,6 +465,7 @@ class BB:
         self.h = Y - y
         self.w = X - x
         self.area = self.h * self.w
+        self.shape = (self.h, self.w)
 
     def __getitem__(self, i):
         return self.bb[i]
@@ -488,6 +493,10 @@ class BB:
         return BB(x + a, y + b, X + a, Y + b)
 
     def remap(self, og_dim: Tuple[int, int], new_dim: Tuple[int, int]):
+        """
+        og_dim = (Height, Width)
+        new_dim = (Height, Width)
+        """
         h, w = og_dim
         H, W = new_dim
         sf_x = H / h
@@ -499,11 +508,11 @@ class BB:
             round(sf_y * self.Y),
         )
 
-    def relative(self, dim: Tuple[int, int]):
+    def relative(self, dim: ("h", "w")):
         h, w = dim
         return BB(self.x / w, self.y / h, self.X / w, self.Y / h)
 
-    def absolute(self, dim: Tuple[int, int]):
+    def absolute(self, dim: ("h", "w")):
         h, w = dim
         return BB(self.x * w, self.y * h, self.X * w, self.Y * h)
 
@@ -540,6 +549,9 @@ class BB:
             dbb = BB(dx, dy, dX, dY)
             return BB([max(0, i + j) for i, j in zip(self, dbb)])
 
+    def shrink_inplace(self):
+        "return a new thing, shrunk"
+
     def add_padding(self, *pad):
         if len(pad) == 4:
             _x, _y, _X, _Y = pad
@@ -549,6 +561,25 @@ class BB:
         x, y, X, Y = self.bb
         return max(0, x - _x), max(0, y - _y), X + _x, Y + _y
 
+    def l2(self, other, xyfactor=(1, 1)):
+        _x_, _y_ = xyfactor
+        other = BB(other)
+        xc, yc = self.xc, self.yc
+        ac, bc = other.xc, other.yc
+        return np.sqrt(_x_ * (xc - ac) ** 2 + _y_ * (yc - bc) ** 2)
+
+    def distances(self, other_bbs, threshold=None, direction=None):
+        other_bbs = bbfy(other_bbs)
+        if direction:
+            assert direction in "x,y,left,right,top,down".split(",")
+            if direction == "x":
+                output = [
+                    (ix, bb, self.l2(bb, xyfactor=(1, 1000))) for (ix, bb) in other_bbs
+                ]
+                return pd.DataFrame(output, columns="ix,bb,dist".split(","))
+            raise NotImplementedError("")
+        return sorted(other_bbs, key=lambda obj: self.l2(obj[1]))
+
 
 def subplots(ims, nc=5, figsize=(5, 5), silent=True, **kwargs):
     if len(ims) == 0:
@@ -640,7 +671,7 @@ def set_logging_level(level):
     logger.add(sys.stderr, level=level)
 
 
-def resize_old(im: np.ndarray, sz: Union[float, Tuple[int, int]]):
+def resize_old(im: np.ndarray, sz: [float, ("H", "W")]):
     h, w = im.shape[:2]
     if isinstance(sz, float):
         frac = sz
@@ -662,7 +693,7 @@ def resize_old(im: np.ndarray, sz: Union[float, Tuple[int, int]]):
 
 
 def resize(
-    im: np.ndarray, sz: Union[float, Tuple[int, int], Tuple[str, Tuple[int, int]]]
+    im: Union[np.ndarray, PIL.Image.Image], sz: [float, ("H", "W"), (str, ("H", "W"))]
 ):
     """Resize an image based on info from sz
     *Aspect ratio is preserved
@@ -682,6 +713,11 @@ def resize(
         >>> _im = resize(im, ('at-most', (40,400))) ; assert _im.shape == (40,80)   #*
         >>> _im = resize(im, ('at-most', (400,40))) ; assert _im.shape == (20,40)   #*
     """
+    if isinstance(im, PIL.Image.Image):
+        im = np.array(im)
+        to_pil = True
+    else:
+        to_pil = False
     h, w = im.shape[:2]
     if isinstance(sz, (tuple, list)) and isinstance(sz[0], str):
         signal, (H, W) = sz
@@ -713,7 +749,10 @@ def resize(
         elif isinstance(W, float):
             W = W * h
     H, W = int(H), int(W)
-    return cv2.resize(im, (W, H))
+    im = cv2.resize(im, (W, H))
+    if to_pil:
+        im = PIL.Image.fromarray(im)
+    return im
 
 
 def pad(im, sz, pad_value=255):
@@ -726,7 +765,7 @@ def pad(im, sz, pad_value=255):
 def xywh2xyXY(bbs):
     if len(bbs) == 4 and isinstance(bbs[0], int):
         x, y, w, h = bbs
-        return BB(x, y, x + w, y + h)
+        return BBox(x, y, x + w, y + h)
     return [xywh2xyXY(bb) for bb in bbs]
 
 
@@ -853,22 +892,27 @@ def shrink_bbs(bbs, eps=0.2):
     ]
 
 
-def batchify(n, bs):
-    n = deepcopy(n)
-    l = []
-    if isinstance(n, int):
-        remaining = n
+from fastcore.basics import patch_to
+
+
+@patch_to(L)
+def first(self):
+    if len(self) > 0:
+        return self[0]
     else:
-        remaining = len(n)
-    while remaining > bs:
-        if isinstance(n, int):
-            l.append(bs)
-        else:
-            l.append(n[:bs])
-            del n[:bs]
-        remaining -= bs
-    if isinstance(n, int):
-        l.append(remaining)
+        return None
+
+
+@patch_to(L)
+def get(self, condition):
+    sublist = self.filter(condition)
+    if len(sublist) > 0:
+        return sublist.first()
     else:
-        l.append(n[:remaining])
-    return l
+        return None
+
+
+@patch_to(L)
+def get_all(self, condition):
+    sublist = self.filter(condition)
+    return sublist
